#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from bigbuild.models import Page
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = 'Creates a new page directory'
    INDEX_TEMPLATE_NAME = 'bigbuild/pages/default_index.html'

    def add_arguments(self, parser):
        """
        Custom arguments for this command
        """
        parser.add_argument('slug', nargs='+', type=str)
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Force overwrite of the page directory if it already exists'
        )

    def set_options(self, *args, **options):
        """
        Configures the command based on user input.
        """
        self.force = options['force']
        self.slug_list = options['slug']

    def create_page(self, slug):
        """
        Returns a new Page object given the slug submitted by the user.
        """
        return Page(slug)

    def create_page_directory(self, page):
        """
        Create a new directory for the provided page.
        """
        page.create_directory(
            force=self.force,
            index_template_name=self.INDEX_TEMPLATE_NAME
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        # Set the options
        self.set_options(*args, **options)

        # Loop through the slugs and create pages
        for slug in self.slug_list:

            # Create the Page
            page = self.create_page(options['slug'])

            # If the directory already exists and we're not forcing creation
            # an error should be thrown.
            if os.path.exists(page.page_directory_path) and not self.force:
                raise CommandError(self.style.ERROR('Page directory already exists'))

            elif os.path.exists(page.archive_dynamic_directory_path) and not self.force:
                raise CommandError(self.style.ERROR('Page directory already exists'))

            # Otherwise we make the page
            else:
                self.create_page_directory(page)

            # Print out the result
            self.stdout.write(self.style.SUCCESS('Created page "%s"' % page))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings
from bakery.management.commands.publish import Command as BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):
        # When in BIGBUILD_BRANCH_BUILD don't delete because we'll be syncing
        # a different subdirectory for each one of our git branches
        if getattr(settings, 'BIGBUILD_BRANCH_BUILD', False):
            options['no_delete'] = True
        super(Command, self).handle(*args, **options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import bigbuild
from django.urls import reverse
from django.http import Http404
from django.conf import settings
from django.views.static import serve
from bigbuild import context_processors
from bigbuild.models import PageList, Page, ArchivedPage
from bigbuild.management.commands.build import Command as Build
from bakery.views import (
    BuildableTemplateView,
    BuildableDetailView,
    Buildable404View,
    BuildableRedirectView
)


class BigBuildMixin(object):
    """
    A class-based view mixin with utilities for our bigbuild pages.
    """
    def process_context(self, context):
        """
        Run bigbuild's custom context processors and return then as part of the provided dictionary.
        """
        context.update(context_processors.bigbuild(self.request))
        return context

    def joinpath(self, *args):
        """
        Returns a joined path with the front slash of the first argument always sliced off.

        Allows for automatically generated URLs from elsewhere in the app be used as relative build paths.
        """
        args = list(args)
        first = args.pop(0)[1:]
        args.insert(0, first)
        return os.path.join(*args)


class Static404View(Buildable404View):
    """
    The 404 page.
    """
    template_name = "bigbuild/404.html"
    build_path = "404.html"


class RobotsView(BuildableTemplateView, BigBuildMixin):
    """
    The robots.txt file.
    """
    template_name = "bigbuild/robots.txt"

    @property
    def build_path(self):
        return self.joinpath(bigbuild.get_base_url(), "robots.txt")


class IndexRedirectView(BuildableRedirectView, BigBuildMixin):
    """
    Redirects the root URL to /projects/
    """
    url = bigbuild.get_base_url()

    @property
    def build_path(self):
        return self.joinpath(reverse('bigbuild-index-redirect'), "index.html")


class PageListView(BuildableTemplateView, BigBuildMixin):
    """
    Renders a homepage with a list of all the pages.
    """
    template_name = "bigbuild/page_list.html"

    @property
    def build_path(self):
        return self.joinpath(reverse('bigbuild-page-list'), "index.html")

    def get_context_data(self):
        PAGE_PUBLICATION_STATUS = getattr(
            settings,
            'PAGE_PUBLICATION_STATUS',
            'working'
        )
        context = {
            'object_list': [p for p in PageList() if p.show_in_feeds],
            'PAGE_PUBLICATION_STATUS': PAGE_PUBLICATION_STATUS
        }
        return self.process_context(context)


class PageDetailView(BuildableDetailView, BigBuildMixin):
    """
    Renders one of the page objects as an HTML response.
    """
    def get_object(self):
        """
        Returns the Page object being rendered by this view.
        """
        # Check if it is a dynamic page, if so return it
        obj = PageList.get_page(self.kwargs.get("slug"), Page)
        if obj:
            return obj
        # Check if it is an archived page, if so return it
        obj = PageList.get_page(self.kwargs.get("slug"), ArchivedPage)
        if obj:
            return obj
        # Otherwise, 404
        raise Http404("No page found with the slug '{}'".format(self.kwargs.get("slug")))

    def get_template_names(self):
        """
        Returns the template paths to use when rendering this object.
        """
        if isinstance(self.object, Page):
            template_dir = self.object.page_directory_path
        elif isinstance(self.object, ArchivedPage):
            template_dir = self.object.archive_static_directory_path
        return [os.path.join(template_dir, "index.html")]

    def get_context_data(self, object=None):
        """
        Returns the context dictionary to use when rending this page.
        """
        context = {
            'metadata': self.object.metadata,
            'object': self.object,
            'STATIC_URL': self.object.get_static_url()
        }
        return self.process_context(context)

    def build_static_directory(self, obj):
        """
        Builds an object's static subdirectory.
        """
        # The location of static files in the dynamic page directory
        source_dir = os.path.join(obj.page_directory_path, 'static')

        # The location in the build directory where we want to copy them
        target_dir = os.path.join(
            bigbuild.get_build_directory(),
            obj.get_static_url()[1:]
        )

        # An internal django-bakery trick to gzip them if we need to
        if settings.BAKERY_GZIP:
            Build().copytree_and_gzip(
                source_dir,
                target_dir
            )
        else:
            # Or a more vanilla way of copying the files with Python
            os.path.exists(target_dir) and shutil.rmtree(target_dir)
            shutil.copytree(source_dir, target_dir)

    def build_object(self, obj):
        """
        Build the provided object
        """
        # If it is a Page object ...
        if isinstance(obj, Page):
            # ... use the default bakery build
            super(PageDetailView, self).build_object(obj)
            # ... and build the static directory as well.
            self.build_static_directory(obj)
        # If it is an ArchivedPage object ...
        elif isinstance(obj, ArchivedPage):
            # ... do a copy and paste from the archive to the build directory
            target = obj.build_directory_path
            os.path.exists(target) and shutil.rmtree(target)
            if settings.BAKERY_GZIP:
                Build().copytree_and_gzip(
                    obj.archive_static_directory_path,
                    target
                )
            else:
                shutil.copytree(obj.archive_static_directory_path, target)

    def build_queryset(self):
        [self.build_object(o) for o in PageList()]


class PageArchiveView(PageDetailView):

    def get_context_data(self, object=None):
        context = super(PageArchiveView, self).get_context_data(object=object)
        context['ARCHIVAL'] = True
        return self.process_context(context)


def page_static_serve(request, slug, path):  # pragma: no cover
    """
    Serves files from the ./static/ subdirectory for every page.
    """
    # Set the path to the file
    path = os.path.join(slug, 'static', path)
    # First test if it can be found in the PAGE_DIR
    try:
        return serve(
            request,
            path,
            document_root=bigbuild.get_page_directory(),
            show_indexes=True
        )
    # If it can't try to fallback to the BIGBUILD_ARCHIVE_DIR
    except Http404:
        return serve(
            request,
            path,
            document_root=os.path.join(bigbuild.get_archive_directory(), 'static'),
            show_indexes=True
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import six
import json
import logging
from django.conf import settings
from collections import Sequence
from bigbuild.exceptions import (
    MissingMetadataWarning,
    MissingRecommendedMetadataWarning
)
from bigbuild.models import Page, ArchivedPage
from dateutil.parser import parse as dateparse
from bigbuild import get_page_directory, get_archive_directory
logger = logging.getLogger(__name__)


class PageList(Sequence):
    """
    A list of all the Page objects in the application.
    """
    def __init__(self):
        # Set the page directories
        self.dynamic_directory = get_page_directory()
        self.archived_directory = get_archive_directory()

        # Set the cache path
        self.archived_cache_path = os.path.join(self.archived_directory, '.cache')

        # Pull the pages
        self.dynamic_pages = self.get_dynamic_pages()
        self.archived_pages = self.get_archived_pages()

        # Create a combined list of live pages and archived pages
        self.pages = []
        self.pages.extend(self.dynamic_pages)
        self.pages.extend(self.archived_pages)

        # Sort first by reverse chron
        self.pages = sorted(
            self.pages,
            key=lambda o: o.pub_date,
            reverse=True
        )

        # Then sort the working pages to the top
        self.pages = sorted(self.pages, key=lambda o: o.is_live())

    def __iter__(self):
        """
        What to use when this PageList is used in a for loop.
        """
        return iter(self.pages)

    def __len__(self):
        """
        Returns the number of pages in this PageList.
        """
        return len(self.pages)

    def __getitem__(self, key):
        """
        Pull items either with a slug key or an integer index.
        """
        if isinstance(key, six.string_types):
            try:
                return [i for i in self.pages if i.slug == key][0]
            except IndexError:
                raise KeyError("No page with that slug was found")
        elif isinstance(key, int):
            try:
                return self.pages[key]
            except IndexError:
                raise IndexError("No page with this key could be found")

    @staticmethod
    def get_page(directory, pagetype):
        """
        Returns a list of Page objects from the provided directory path.
        """
        # Ignore any names in our blacklist
        if directory in getattr(settings, 'BIGBUILD_PAGE_BLACKLIST', ['.DS_Store']):
            return None

        # Create a Page object from the directory slug
        page = pagetype(directory)

        # Verify it has frontmatter and is a qualified page
        if not os.path.exists(page.frontmatter_path):
            # If it doesn't have frontmatter broadcast a warning
            # to the developer and skip this directory.
            warning = MissingMetadataWarning(page)
            logger.warn(warning)
            return None

        # Sync in the metadata from the filesystem to the object
        page.sync_frontmatter()

        # Make sure the page is valid
        if not page.is_metadata_valid():
            raise ValueError("Metadata is not valid for %s" % page)

        # Make sure the page has recommended metadata
        # ... if it's ready to publish
        if page.pub_status in ['live', 'pending']:
            if not page.has_recommended_metadata():
                logger.warn(MissingRecommendedMetadataWarning(page))

        return page

    def get_dynamic_pages(self):
        """
        Returns a list of Page objects ready to be built
        in this environment.
        """
        logger.debug("Retrieving dynamic page list")
        page_list = []
        for d in os.listdir(self.dynamic_directory):
            page = self.get_page(d, Page)
            if page and page.should_build():
                page_list.append(page)
        logger.debug("{} dynamic pages retrieved".format(len(page_list)))
        return page_list

    def get_archived_pages(self):
        """
        Returns a list of ArchivedPage objects ready to be built
        in this environment.
        """
        # Pull the cached data if it exists
        if os.path.exists(self.archived_cache_path):
            logger.debug("Loading cached archived page list")
            with open(self.archived_cache_path, 'r') as f:
                page_list = []
                dict_list = json.load(f)['archived_pages']
                for d in dict_list:
                    d['pub_date'] = dateparse(d['pub_date'])
                    page = ArchivedPage(**d)
                    if page.should_build():
                        page_list.append(page)
        # Otherwise get them from the YAML
        else:
            logger.debug("Retrieving YAML archived page list")
            page_list = []
            for d in os.listdir(os.path.join(self.archived_directory, 'static')):
                page = self.get_page(d, ArchivedPage)
                if page and page.should_build():
                    page_list.append(page)

        # Log and return
        logger.debug("{} archived pages retrieved".format(len(page_list)))
        return page_list
